package eu.mhsl.minecraft.bungeequeue;

import eu.mhsl.minecraft.bungeequeue.command.MaxPlayers;
import eu.mhsl.minecraft.bungeequeue.listener.PlayerJoin;
import eu.mhsl.minecraft.bungeequeue.scheduler.MoveToTarget;
import eu.mhsl.minecraft.bungeequeue.scheduler.PlayersWaitingCheck;
import eu.mhsl.minecraft.bungeequeue.scheduler.QueueInfoMessager;
import net.md_5.bungee.api.ProxyServer;
import net.md_5.bungee.api.config.ServerInfo;
import net.md_5.bungee.api.connection.ProxiedPlayer;
import net.md_5.bungee.api.plugin.Plugin;
import net.md_5.bungee.config.Configuration;
import net.md_5.bungee.config.ConfigurationProvider;
import net.md_5.bungee.config.YamlConfiguration;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class BungeeQueue extends Plugin {
    public static Plugin plugin;
    public static ServerInfo queueServer;
    public static ServerInfo targetServer;

    public static int maxPlayers = 1; //mutable per command and config

    public static Configuration conf;
    public static File dataFolder;

    public static HashMap<ProxiedPlayer, Integer> waitingSince = new HashMap<>();
    public static List<ProxiedPlayer> list = new ArrayList<>();

    @Override
    public void onEnable() {
        plugin = this;
        dataFolder = getDataFolder();

        try {
            if(!dataFolder.exists()) dataFolder.mkdir();
            File configFile = new File(dataFolder, "config.yml");
            if(!configFile.exists()) {
                try(InputStream in = getResourceAsStream("config.example.yml")) {
                    Files.copy(in, configFile.toPath());
                } catch(IOException e) {
                    e.printStackTrace();
                }
            }

            conf = ConfigurationProvider.getProvider(YamlConfiguration.class).load(new File(dataFolder, "config.yml"));
            maxPlayers = conf.getInt("maxPlayers");
        } catch(IOException e) {
            e.printStackTrace();
        }

        ProxyServer instance = ProxyServer.getInstance();

        queueServer = instance.getServerInfo(conf.getString("queueServer"));
        targetServer = instance.getServerInfo(conf.getString("targetServer"));

        instance.getPluginManager().registerListener(this, new PlayerJoin());
        instance.getPluginManager().registerCommand(this, new MaxPlayers());

        instance.getScheduler().schedule(this, new MoveToTarget(), 5, 1, TimeUnit.SECONDS);
        instance.getScheduler().schedule(this, new QueueInfoMessager(), 1, 2, TimeUnit.SECONDS);
        instance.getScheduler().schedule(this, new PlayersWaitingCheck(), 10, 10, TimeUnit.SECONDS);

    }

    public void onDisable() {
        ProxyServer.getInstance().getPluginManager().unregisterListeners(this);
        ProxyServer.getInstance().getPluginManager().unregisterCommands(this);
        saveConfig();
    }

    public static boolean saveConfig() {
        try {
            ConfigurationProvider.getProvider(YamlConfiguration.class).save(conf, new File(dataFolder, "config.yml"));
            return true;
        } catch(IOException e) {
            e.printStackTrace();
            return false;
        }
    }

}
